#***************************************************************************
#*
#**  File     :  /lua/ai/AIEconomicBuilders.lua
#**
#**  Summary  : Default economic builders for skirmish
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************

local BBTmplFile = '/lua/basetemplates.lua'
local BuildingTmpl = 'BuildingTemplates'
local BaseTmpl = 'BaseTemplates'
local ExBaseTmpl = 'ExpansionBaseTemplates'
local Adj2x2Tmpl = 'Adjacency2x2'
local UCBC = '/lua/editor/UnitCountBuildConditions.lua'
local MIBC = '/lua/editor/MiscBuildConditions.lua'
local MABC = '/lua/editor/MarkerBuildConditions.lua'
local IBC = '/lua/editor/InstantBuildConditions.lua'
local OAUBC = '/lua/editor/OtherArmyUnitCountBuildConditions.lua'
local EBC = '/lua/editor/EconomyBuildConditions.lua'
local PCBC = '/lua/editor/PlatoonCountBuildConditions.lua'
local SAI = '/lua/ScenarioPlatoonAI.lua'
local TBC = '/lua/editor/ThreatBuildConditions.lua'
local PlatoonFile = '/lua/platoon.lua'
local SIBC = '/lua/editor/SorianInstantBuildConditions.lua'
local SBC = '/mods/BlackopsACUs/lua/SorianBuildConditions.lua'
local BOUC = '/mods/BlackopsACUs/lua/EXBlackopsUpgradeConditions.lua'

do
####################
########## Overwriting Duncanes's To Prevent Issues
####################
BuilderGroup { #DUNCAN - added group
    BuilderGroupName = 'ACUUpgrades - Gun improvements',
    BuildersType = 'EngineerBuilder',
    # UEF
    Builder {
        BuilderName = 'UEF CDR Upgrade HeavyAntiMatter',
        PlatoonTemplate = 'CommanderEnhance',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'HeavyAntiMatterCannon', true }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'HeavyAntiMatterCannon' },
        },
        
    },
	# Aeon
    Builder {
        BuilderName = 'Aeon CDR Upgrade Crysalis',
        PlatoonTemplate = 'CommanderEnhance',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'CrysalisBeam', true }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
			TimeBetweenEnhancements = 20,
            Enhancement = { 'HeatSink', 'CrysalisBeam'},
        },
    },
	# Cybran
    Builder {
        BuilderName = 'Cybran CDR Upgrade CoolingUpgrade',
        PlatoonTemplate = 'CommanderEnhance',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'CoolingUpgrade', true }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'CoolingUpgrade'},
        },
       
    },
	# Seraphim
    Builder {
        BuilderName = 'Seraphim CDR Upgrade RateOfFire',
        PlatoonTemplate = 'CommanderEnhance', 
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'RateOfFire', true }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'RateOfFire' },
        },
		
    },
}

BuilderGroup {
    BuilderGroupName = 'ACUUpgrades - Tech 2 Engineering',
    BuildersType = 'EngineerBuilder',
    # UEF
    Builder {
        BuilderName = 'UEF CDR Upgrade Adv Engi',
        PlatoonTemplate = 'CommanderEnhance',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'AdvancedEngineering', true }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'LeftPod', 'RightPod', 'AdvancedEngineering' },
        },
    },
	# Aeon
    Builder {
        BuilderName = 'Aeon CDR Upgrade Adv Engi, replacing Crysalis',
        PlatoonTemplate = 'CommanderEnhance',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'AdvancedEngineering', true }},
				{ SBC, 'CmdrHasUpgrade', { 'CrysalisBeam', true }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'CrysalisBeamRemove', 'AdvancedEngineering'},
        },
        
    },
	Builder {
        BuilderName = 'Aeon CDR Upgrade Adv Engi',
        PlatoonTemplate = 'CommanderEnhance',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'AdvancedEngineering', true }},
				{ SBC, 'CmdrHasUpgrade', { 'CrysalisBeam', false }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'AdvancedEngineering'},
        },
        
    },
	# Cybran
    Builder {
        BuilderName = 'Cybran CDR Upgrade Adv Engi, replacing cooling',
        PlatoonTemplate = 'CommanderEnhance',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'AdvancedEngineering', true }},
				{ SBC, 'CmdrHasUpgrade', { 'CoolingUpgrade', true}},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'CoolingUpgradeRemove','AdvancedEngineering'},
        },
        
    },
	 Builder {
        BuilderName = 'Cybran CDR Upgrade Adv Engi',
        PlatoonTemplate = 'CommanderEnhance',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'AdvancedEngineering', true }},
				{ SBC, 'CmdrHasUpgrade', { 'CoolingUpgrade', false }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'AdvancedEngineering'},
        },
        
    },
	# Seraphim
    Builder {
        BuilderName = 'Seraphim CDR Upgrade Adv Engi',
        PlatoonTemplate = 'CommanderEnhance',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'AdvancedEngineering', true }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'AdvancedEngineering' },
        },
		
    },
}

BuilderGroup {
    BuilderGroupName = 'ACUUpgrades - Shields',
    BuildersType = 'EngineerBuilder',
    # UEF
    Builder {
        BuilderName = 'UEF CDR Upgrade Shield, replaceing right pod',
        PlatoonTemplate = 'CommanderEnhance',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'Shield', true }},
				{ SBC, 'CmdrHasUpgrade', { 'RightPod', true }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'RightPodRemove', 'Shield' },
        },
    },
	Builder {
        BuilderName = 'UEF CDR Upgrade Shield',
        PlatoonTemplate = 'CommanderEnhance',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'Shield', true }},
				{ SBC, 'CmdrHasUpgrade', { 'RightPod', false }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'Shield' },
        },
    },
	# Aeon
    Builder {
        BuilderName = 'Aeon CDR Upgrade Shield',
        PlatoonTemplate = 'CommanderEnhance',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'Shield', true }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'Shield' },
        },
        
    },
	# Cybran
    Builder {
        BuilderName = 'Cybran CDR Upgrade Stealth and Cloak',
        PlatoonTemplate = 'CommanderEnhance',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'CloakingGenerator', true }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'StealthGenerator', 'CloakingGenerator', 'MicrowaveLaserGenerator'},
        },
        
    },
	# Seraphim
    Builder {
        BuilderName = 'Seraphim CDR Upgrade AdvancedRegenAura',
        PlatoonTemplate = 'CommanderEnhance',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'AdvancedRegenAura', true }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'RegenAura', 'AdvancedRegenAura' },
        },
		
    },
	Builder {
        BuilderName = 'Seraphim CDR Upgrade AdvancedRegenAura, replace rate of fire',
        PlatoonTemplate = 'CommanderEnhance',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'AdvancedRegenAura', true }},
				{ SBC, 'CmdrHasUpgrade', { 'RateOfFire', true }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'RateOfFireRemove', 'RegenAura', 'AdvancedRegenAura' },
        },
    },
}

BuilderGroup {
    BuilderGroupName = 'SorianACUUpgrades',
    BuildersType = 'EngineerBuilder', #'PlatoonFormBuilder',

####################
########## Overwriting Sorian's To Prevent Issues
####################

    # UEF
    Builder {
        BuilderName = 'Sorian UEF CDR Upgrade AdvEng - Pods',
        PlatoonTemplate = 'CommanderEnhanceSorian',
        BuilderConditions = {
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'FACTORY TECH2, FACTORY TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'MASSEXTRACTION TECH2, MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'ENERGYPRODUCTION TECH2, ENERGYPRODUCTION TECH3' }},
                #{ EBC, 'GreaterThanEconStorageRatio', { 0.25, 0.25}},
				{ SIBC, 'GreaterThanEconEfficiencyOverTime', { 0.9, 1.2 }},
				{ SBC, 'CmdrHasUpgrade', { 'T3Engineering', true }},
				{ SBC, 'CmdrHasUpgrade', { 'Shield', false }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        Priority = 900,
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'LeftPod', 'RightPod', 'AdvancedEngineering', 'T3Engineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR Upgrade T3 Eng - Shields',
        PlatoonTemplate = 'CommanderEnhanceSorian',
        BuilderConditions = {
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'FACTORY TECH2, FACTORY TECH3'}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'MASSEXTRACTION TECH2, MASSEXTRACTION TECH3'}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 0, 'ENERGYPRODUCTION TECH3'}},
                #{ EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
				{ SIBC, 'GreaterThanEconEfficiencyOverTime', { 0.9, 1.2 }},
				{ SBC, 'CmdrHasUpgrade', { 'T3Engineering', true }},
				{ SBC, 'CmdrHasUpgrade', { 'Shield', false }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        Priority = 900,
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'ResourceAllocation', 'RightPodRemove', 'Shield' },
        },
    },

    # Aeon
    Builder {
        BuilderName = 'Sorian Aeon CDR Upgrade AdvEng - Resource - Crysalis',
        PlatoonTemplate = 'CommanderEnhanceSorian',
        BuilderConditions = {
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'FACTORY TECH2, FACTORY TECH3'}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'MASSEXTRACTION TECH2, MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'ENERGYPRODUCTION TECH2, ENERGYPRODUCTION TECH3' }},
                #{ EBC, 'GreaterThanEconStorageRatio', { 0.25, 0.25}},
				{ SIBC, 'GreaterThanEconEfficiencyOverTime', { 0.9, 1.2 }},
				{ SBC, 'CmdrHasUpgrade', { 'HeatSink', false }},
				{ SBC, 'CmdrHasUpgrade', { 'T3Engineering', true }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        Priority = 900,
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'AdvancedEngineering', 'T3Engineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR Upgrade T3 Eng - ResourceAdv - EnhSensor',
        PlatoonTemplate = 'CommanderEnhanceSorian',
        BuilderConditions = {
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'FACTORY TECH2, FACTORY TECH3'}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'MASSEXTRACTION TECH2, MASSEXTRACTION TECH3'}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 0, 'ENERGYPRODUCTION TECH3'}},
                #{ EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
				{ SIBC, 'GreaterThanEconEfficiencyOverTime', { 0.9, 1.2 }},
				{ SBC, 'CmdrHasUpgrade', { 'T3Engineering', true }},
				{ SBC, 'CmdrHasUpgrade', { 'HeatSink', false }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        Priority = 900,
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'Shield', 'HeatSink' },
        },
    },
    
    # Cybran
    Builder {
        BuilderName = 'Sorian Cybran CDR Upgrade AdvEng - Laser Gen',
        PlatoonTemplate = 'CommanderEnhanceSorian',
        BuilderConditions = {
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'FACTORY TECH2, FACTORY TECH3'}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'MASSEXTRACTION TECH2, MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'ENERGYPRODUCTION TECH2, ENERGYPRODUCTION TECH3' }},
                #{ EBC, 'GreaterThanEconStorageRatio', { 0.25, 0.25}},
				{ SIBC, 'GreaterThanEconEfficiencyOverTime', { 0.9, 1.2 }},
				{ SBC, 'CmdrHasUpgrade', { 'T3Engineering', true }},
				{ SBC, 'CmdrHasUpgrade', { 'MicrowaveLaserGenerator', false }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        Priority = 900,
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'StealthGenerator', 'AdvancedEngineering', 'T3Engineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR Upgrade T3 Eng - Resource',
        PlatoonTemplate = 'CommanderEnhanceSorian',
        BuilderConditions = {
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'FACTORY TECH2, FACTORY TECH3'}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'MASSEXTRACTION TECH2, MASSEXTRACTION TECH3'}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 0, 'ENERGYPRODUCTION TECH3'}},
                #{ EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
				{ SIBC, 'GreaterThanEconEfficiencyOverTime', { 0.9, 1.2 }},
				{ SBC, 'CmdrHasUpgrade', { 'T3Engineering', true }},
				{ SBC, 'CmdrHasUpgrade', { 'MicrowaveLaserGenerator', false }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        Priority = 0, #900,
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'MicrowaveLaserGenerator' },
        },
    },
	
    # Seraphim
    Builder {
        BuilderName = 'Sorian Seraphim CDR Upgrade AdvEng - Resource - Crysalis',
        PlatoonTemplate = 'CommanderEnhanceSorian',
        BuilderConditions = {
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'FACTORY TECH2, FACTORY TECH3'}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'MASSEXTRACTION TECH2, MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'ENERGYPRODUCTION TECH2, ENERGYPRODUCTION TECH3' }},
                #{ EBC, 'GreaterThanEconStorageRatio', { 0.25, 0.25}},
				{ SIBC, 'GreaterThanEconEfficiencyOverTime', { 0.9, 1.2 }},
				{ SBC, 'CmdrHasUpgrade', { 'AdvancedRegenAura', false }},
				{ SBC, 'CmdrHasUpgrade', { 'T3Engineering', true }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        Priority = 900,
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'AdvancedEngineering', 'RegenAura', 'T3Engineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR Upgrade T3 Eng - ResourceAdv - EnhSensor',
        PlatoonTemplate = 'CommanderEnhanceSorian',
        BuilderConditions = {
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'FACTORY TECH2, FACTORY TECH3'}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'MASSEXTRACTION TECH2, MASSEXTRACTION TECH3'}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 0, 'ENERGYPRODUCTION TECH3'}},
                #{ EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
				{ SIBC, 'GreaterThanEconEfficiencyOverTime', { 0.9, 1.2 }},
				{ SBC, 'CmdrHasUpgrade', { 'T3Engineering', true }},
				{ SBC, 'CmdrHasUpgrade', { 'AdvancedRegenAura', false }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        Priority = 900,
        BuilderType = 'Any',
		PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderData = {
            Enhancement = { 'AdvancedRegenAura' },
        },
    },
   
####################
########## Exaviers UEF ACUs
####################
##### Tier One
##########
   Builder {
        BuilderName = 'Sorian UEF CDR EXImprovedEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 2000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', false}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'TECH2' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'MASSEXTRACTION' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'ENERGYPRODUCTION' }},
				{ BOUC, 'CanUpgrade', { 'EXImprovedEngineering' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedEngineering' },
        },
    },
   Builder {
        BuilderName = 'Sorian UEF CDR EXCombatEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 2000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', false}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'TECH2' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'MASSEXTRACTION' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'ENERGYPRODUCTION' }},
				{ BOUC, 'CanUpgrade', { 'EXImprovedEngineering' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXCombatEngineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXClusterMisslePack',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1600,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXActiveShielding', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXShieldBattery' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXClusterMisslePack' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXShieldBattery',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1600,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXClusterMisslePack', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalMisslePack', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalNukeSubstitution', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXShieldBattery' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXShieldBattery' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXClusterMisslePack v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1600,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXActiveShielding', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXShieldBattery' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXClusterMisslePack' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXShieldBattery v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1600,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXClusterMisslePack', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalMisslePack', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalNukeSubstitution', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXShieldBattery' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXShieldBattery' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXAntiMatterCannon',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1700,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXGattlingEnergyCannon', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXEnergyShellHardener', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXAntiMatterCannon' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXAntiMatterCannon' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXGattlingEnergyCannon',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1700,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXAntiMatterCannon', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedContainmentBottle', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXAntiMatterCannon' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXGattlingEnergyCannon' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXAntiMatterCannon v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1700,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXGattlingEnergyCannon', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXEnergyShellHardener', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXAntiMatterCannon' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXAntiMatterCannon' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXGattlingEnergyCannon v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1700,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXAntiMatterCannon', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedContainmentBottle', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXAntiMatterCannon' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXGattlingEnergyCannon' },
        },
    },
##########
##### Tier Two
##########
    Builder {
        BuilderName = 'Sorian UEF CDR EXAdvancedEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1500,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}}, 
				{ UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'MASSEXTRACTION TECH2, MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'ENERGYPRODUCTION TECH2, ENERGYPRODUCTION TECH3' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 5, 'TECH2, TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXAdvancedEngineering' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXAdvancedEngineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXAssaultEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1500,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}}, 
				{ UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'MASSEXTRACTION TECH2, MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'ENERGYPRODUCTION TECH2, ENERGYPRODUCTION TECH3' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 5, 'TECH2, TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXAdvancedEngineering' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXAssaultEngineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXTacticalMisslePack',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1200,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXActiveShielding', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXClusterMisslePack', true}},
				{ BOUC, 'CanUpgrade', { 'EXActiveShielding' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXTacticalMisslePack' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXActiveShielding',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1200,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXClusterMisslePack', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalMisslePack', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalNukeSubstitution', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXShieldBattery', true}},
				{ BOUC, 'CanUpgrade', { 'EXActiveShielding' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXActiveShielding' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXTacticalMisslePack v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1200,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXActiveShielding', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXClusterMisslePack', true}},
				{ BOUC, 'CanUpgrade', { 'EXActiveShielding' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXTacticalMisslePack' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXActiveShielding v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1200,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXClusterMisslePack', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalMisslePack', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalNukeSubstitution', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXShieldBattery', true}},
				{ BOUC, 'CanUpgrade', { 'EXActiveShielding' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXActiveShielding' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXImprovedContainmentBottle',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1300,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXGattlingEnergyCannon', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXEnergyShellHardener', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAntiMatterCannon', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedContainmentBottle' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2, TECH3' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedContainmentBottle' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXImprovedCoolingSystem',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1300,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXAntiMatterCannon', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedContainmentBottle', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXGattlingEnergyCannon', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedContainmentBottle' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2, TECH3' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedCoolingSystem' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXImprovedContainmentBottle v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1300,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXGattlingEnergyCannon', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXEnergyShellHardener', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAntiMatterCannon', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedContainmentBottle' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2, TECH3' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedContainmentBottle' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXImprovedCoolingSystem v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1300,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXAntiMatterCannon', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedContainmentBottle', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXGattlingEnergyCannon', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedContainmentBottle' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2, TECH3' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedCoolingSystem' },
        },
    },
##########
##### Tier Three
##########
    Builder {
        BuilderName = 'Sorian UEF CDR EXExperimentalEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1100,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'ENERGYPRODUCTION TECH3' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 5, 'TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXExperimentalEngineering' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXExperimentalEngineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXApocolypticEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1100,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'ENERGYPRODUCTION TECH3' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 5, 'TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXExperimentalEngineering' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXApocolypticEngineering' },
        },
    },
   	Builder {
        BuilderName = 'Sorian UEF CDR EXTacticalNukeSubstitution',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 900,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXActiveShielding', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalMisslePack', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedShieldBattery' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXTacticalNukeSubstitution' },
        },
    },
   	Builder {
        BuilderName = 'Sorian UEF CDR EXImprovedShieldBattery',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 900,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXClusterMisslePack', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalMisslePack', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalNukeSubstitution', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXActiveShielding', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedShieldBattery' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedShieldBattery' },
        },
    },
   	Builder {
        BuilderName = 'Sorian UEF CDR EXTacticalNukeSubstitution v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 900,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXActiveShielding', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalMisslePack', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedShieldBattery' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXTacticalNukeSubstitution' },
        },
    },
   	Builder {
        BuilderName = 'Sorian UEF CDR EXImprovedShieldBattery v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 900,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXClusterMisslePack', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalMisslePack', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalNukeSubstitution', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXActiveShielding', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedShieldBattery' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedShieldBattery' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXPowerBooster',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXGattlingEnergyCannon', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXEnergyShellHardener', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedContainmentBottle', true}},
				{ BOUC, 'CanUpgrade', { 'EXPowerBooster' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH3' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXPowerBooster' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXEnergyShellHardener',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXAntiMatterCannon', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedContainmentBottle', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', true}},
				{ BOUC, 'CanUpgrade', { 'EXPowerBooster' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH3' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXEnergyShellHardener' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXPowerBooster v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXGattlingEnergyCannon', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXEnergyShellHardener', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedContainmentBottle', true}},
				{ BOUC, 'CanUpgrade', { 'EXPowerBooster' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH3' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXPowerBooster' },
        },
    },
    Builder {
        BuilderName = 'Sorian UEF CDR EXEnergyShellHardener v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXAntiMatterCannon', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedContainmentBottle', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', true}},
				{ BOUC, 'CanUpgrade', { 'EXPowerBooster' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH3' }},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXEnergyShellHardener' },
        },
    },
    
####################
########## Exaviers Aeon ACUs
####################
##### Tier One
##########
   Builder {
        BuilderName = 'Sorian Aeon CDR EXImprovedEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 2000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', false}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'TECH2' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'MASSEXTRACTION' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'ENERGYPRODUCTION' }},
				{ BOUC, 'CanUpgrade', { 'EXImprovedEngineering' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedEngineering' },
        },
    },
   Builder {
        BuilderName = 'Sorian Aeon CDR EXCombatEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 2000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', false}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'TECH2' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'MASSEXTRACTION' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'ENERGYPRODUCTION' }},
				{ BOUC, 'CanUpgrade', { 'EXImprovedEngineering' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXCombatEngineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXMaelstromQuantum',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1600,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXActiveShielding', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXShieldBattery' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXMaelstromQuantum' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXShieldBattery',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1600,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXMaelstromQuantum', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXFieldExpander', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXQuantumInstability', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXShieldBattery' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXShieldBattery' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXMaelstromQuantum v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1600,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXActiveShielding', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXShieldBattery' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXMaelstromQuantum' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXShieldBattery v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1600,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXMaelstromQuantum', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXFieldExpander', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXQuantumInstability', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXShieldBattery' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXShieldBattery' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXArtilleryMiasma',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1700,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXBeamPhason', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXArtilleryMiasma' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXArtilleryMiasma' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXBeamPhason',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1700,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXArtilleryMiasma', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedShells', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedReloader', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXArtilleryMiasma' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXBeamPhason' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXArtilleryMiasma v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1700,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXBeamPhason', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXArtilleryMiasma' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXArtilleryMiasma' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXBeamPhason v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1700,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXArtilleryMiasma', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedShells', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedReloader', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXArtilleryMiasma' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXBeamPhason' },
        },
    },
##########
##### Tier Two
##########
    Builder {
        BuilderName = 'Sorian Aeon CDR EXAdvancedEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1500,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}}, 
				{ UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'MASSEXTRACTION TECH2, MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'ENERGYPRODUCTION TECH2, ENERGYPRODUCTION TECH3' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 5, 'TECH2, TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXAdvancedEngineering' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXAdvancedEngineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXAssaultEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1500,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}}, 
				{ UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'MASSEXTRACTION TECH2, MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'ENERGYPRODUCTION TECH2, ENERGYPRODUCTION TECH3' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 5, 'TECH2, TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXAdvancedEngineering' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXAssaultEngineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXFieldExpander',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1200,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXActiveShielding', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXMaelstromQuantum', true}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'ENERGYPRODUCTION TECH2, ENERGYPRODUCTION TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXActiveShielding' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXFieldExpander' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXActiveShielding',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1200,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXMaelstromQuantum', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXFieldExpander', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXQuantumInstability', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXShieldBattery', true}},
				{ BOUC, 'CanUpgrade', { 'EXActiveShielding' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXActiveShielding' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXFieldExpander v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1200,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXActiveShielding', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXMaelstromQuantum', true}},
				{ BOUC, 'CanUpgrade', { 'EXActiveShielding' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXFieldExpander' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXActiveShielding v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1200,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXMaelstromQuantum', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXFieldExpander', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXQuantumInstability', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXShieldBattery', true}},
				{ BOUC, 'CanUpgrade', { 'EXActiveShielding' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXActiveShielding' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXAdvancedShells',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1300,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXBeamPhason', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXArtilleryMiasma', true}},
                { BOUC, 'CanUpgrade', { 'EXAdvancedShells' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2, TECH3' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXAdvancedShells' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXImprovedCoolingSystem',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1300,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXArtilleryMiasma', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedShells', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedReloader', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXBeamPhason', true}},
                { BOUC, 'CanUpgrade', { 'EXAdvancedShells' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2, TECH3' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedCoolingSystem' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXAdvancedShells v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1300,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXBeamPhason', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXArtilleryMiasma', true}},
				{ BOUC, 'CanUpgrade', { 'EXAdvancedShells' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2, TECH3' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXAdvancedShells' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXImprovedCoolingSystem v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1300,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXArtilleryMiasma', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedShells', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedReloader', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXBeamPhason', true}},
				{ BOUC, 'CanUpgrade', { 'EXAdvancedShells' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2, TECH3' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedCoolingSystem' },
        },
    },
##########
##### Tier Three
##########
    Builder {
        BuilderName = 'Sorian Aeon CDR EXExperimentalEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1100,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'ENERGYPRODUCTION TECH3' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 5, 'TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXExperimentalEngineering' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXExperimentalEngineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXApocolypticEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1100,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'ENERGYPRODUCTION TECH3' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 5, 'TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXExperimentalEngineering' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXApocolypticEngineering' },
        },
    },
   	Builder {
        BuilderName = 'Sorian Aeon CDR EXQuantumInstability',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 900,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXActiveShielding', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXFieldExpander', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedShieldBattery' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXQuantumInstability' },
        },
    },
   	Builder {
        BuilderName = 'Sorian Aeon CDR EXImprovedShieldBattery',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 900,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXMaelstromQuantum', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXFieldExpander', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXQuantumInstability', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXActiveShielding', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedShieldBattery' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedShieldBattery' },
        },
    },
   	Builder {
        BuilderName = 'Sorian Aeon CDR EXQuantumInstability v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 900,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXActiveShielding', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedShieldBattery', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXFieldExpander', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedShieldBattery' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXQuantumInstability' },
        },
    },
   	Builder {
        BuilderName = 'Sorian Aeon CDR EXImprovedShieldBattery v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 900,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXMaelstromQuantum', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXFieldExpander', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXQuantumInstability', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXActiveShielding', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedShieldBattery' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedShieldBattery' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXImprovedReloader',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXBeamPhason', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedShells', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedReloader' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH3' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedReloader' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXPowerBooster',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXArtilleryMiasma', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedShells', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedReloader', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedReloader' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH3' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXPowerBooster' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXImprovedReloader v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXBeamPhason', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedShells', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedReloader' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH3' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedReloader' },
        },
    },
    Builder {
        BuilderName = 'Sorian Aeon CDR EXPowerBooster v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXArtilleryMiasma', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedShells', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedReloader', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedReloader' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH3' }},
                { MIBC, 'FactionIndex', {2, 2}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXPowerBooster' },
        },
    },

####################
########## Exaviers Cybran ACUs
####################
##### Tier One
##########
   Builder {
        BuilderName = 'Sorian Cybran CDR EXImprovedEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 2000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', false}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'TECH2' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'MASSEXTRACTION' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'ENERGYPRODUCTION' }},
				{ BOUC, 'CanUpgrade', { 'EXImprovedEngineering' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedEngineering' },
        },
    },
   Builder {
        BuilderName = 'Sorian Cybran CDR EXCombatEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 2000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', false}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'TECH2' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'MASSEXTRACTION' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'ENERGYPRODUCTION' }},
				{ BOUC, 'CanUpgrade', { 'EXImprovedEngineering' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXCombatEngineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXElectronicsEnhancment',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1600,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXArmorPlating', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXStructuralIntegrity', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCompositeMaterials', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXArmorPlating' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXElectronicsEnhancment' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXArmorPlating',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1600,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXElectronicsEnhancment', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXElectronicCountermeasures', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCloakingSubsystems', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXArmorPlating' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXArmorPlating' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXElectronicsEnhancment v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1600,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXArmorPlating', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXStructuralIntegrity', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCompositeMaterials', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXArmorPlating' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXElectronicsEnhancment' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXArmorPlating v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1600,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXElectronicsEnhancment', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXElectronicCountermeasures', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCloakingSubsystems', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXArmorPlating' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXArmorPlating' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXEMPArray',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1700,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXMasor', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEmitterArray', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXMasor' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXEMPArray' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXMasor',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1700,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXEMPArray', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCapacitors', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXMasor' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXMasor' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXEMPArray v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1700,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXMasor', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEmitterArray', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXMasor' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXEMPArray' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXMasor v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1700,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXEMPArray', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCapacitors', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXMasor' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXMasor' },
        },
    },
##########
##### Tier Two
##########
    Builder {
        BuilderName = 'Sorian Cybran CDR EXAdvancedEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1500,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}}, 
				{ UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'MASSEXTRACTION TECH2, MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'ENERGYPRODUCTION TECH2, ENERGYPRODUCTION TECH3' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 5, 'TECH2, TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXAdvancedEngineering' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXAdvancedEngineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXAssaultEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1500,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}}, 
				{ UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'MASSEXTRACTION TECH2, MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'ENERGYPRODUCTION TECH2, ENERGYPRODUCTION TECH3' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 5, 'TECH2, TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXAdvancedEngineering' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXAssaultEngineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXElectronicCountermeasures',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1200,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXArmorPlating', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXStructuralIntegrity', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCompositeMaterials', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXElectronicsEnhancment', true}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'ENERGYPRODUCTION TECH2, ENERGYPRODUCTION TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXStructuralIntegrity' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXElectronicCountermeasures' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXStructuralIntegrity',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1200,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXElectronicsEnhancment', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXElectronicCountermeasures', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCloakingSubsystems', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXArmorPlating', true}},
				{ BOUC, 'CanUpgrade', { 'EXStructuralIntegrity' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXStructuralIntegrity' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXElectronicCountermeasures v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1200,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXArmorPlating', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXStructuralIntegrity', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCompositeMaterials', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXElectronicsEnhancment', true}},
				{ BOUC, 'CanUpgrade', { 'EXStructuralIntegrity' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXElectronicCountermeasures' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXStructuralIntegrity v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1200,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXElectronicsEnhancment', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXElectronicCountermeasures', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCloakingSubsystems', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXArmorPlating', true}},
				{ BOUC, 'CanUpgrade', { 'EXStructuralIntegrity' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXStructuralIntegrity' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXImprovedCapacitors',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1300,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXMasor', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEmitterArray', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXEMPArray', true}},
                { BOUC, 'CanUpgrade', { 'EXImprovedCapacitors' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2, TECH3' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedCapacitors' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXImprovedCoolingSystem',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1300,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXEMPArray', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCapacitors', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXMasor', true}},
                { BOUC, 'CanUpgrade', { 'EXImprovedCapacitors' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2, TECH3' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedCoolingSystem' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXImprovedCapacitors v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1300,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXMasor', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEmitterArray', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXEMPArray', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedCapacitors' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2, TECH3' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedCapacitors' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXImprovedCoolingSystem v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1300,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXEMPArray', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCapacitors', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXMasor', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedCapacitors' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2, TECH3' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedCoolingSystem' },
        },
    },
##########
##### Tier Three
##########
    Builder {
        BuilderName = 'Sorian Cybran CDR EXExperimentalEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1100,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'ENERGYPRODUCTION TECH3' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 5, 'TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXExperimentalEngineering' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXExperimentalEngineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXApocolypticEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1100,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'ENERGYPRODUCTION TECH3' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 5, 'TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXExperimentalEngineering' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXApocolypticEngineering' },
        },
    },
   	Builder {
        BuilderName = 'Sorian Cybran CDR EXCloakingSubsystems',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 900,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXArmorPlating', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXStructuralIntegrity', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCompositeMaterials', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXElectronicCountermeasures', true}},
				{ BOUC, 'CanUpgrade', { 'EXCompositeMaterials' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXCloakingSubsystems' },
        },
    },
   	Builder {
        BuilderName = 'Sorian Cybran CDR EXCompositeMaterials',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 900,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXElectronicsEnhancment', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXElectronicCountermeasures', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCloakingSubsystems', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXStructuralIntegrity', true}},
				{ BOUC, 'CanUpgrade', { 'EXCompositeMaterials' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXCompositeMaterials' },
        },
    },
   	Builder {
        BuilderName = 'Sorian Cybran CDR EXCloakingSubsystems v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 900,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXArmorPlating', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXStructuralIntegrity', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCompositeMaterials', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXElectronicCountermeasures', true}},
				{ BOUC, 'CanUpgrade', { 'EXCompositeMaterials' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXCloakingSubsystems' },
        },
    },
   	Builder {
        BuilderName = 'Sorian Cybran CDR EXCompositeMaterials v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 900,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXElectronicsEnhancment', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXElectronicCountermeasures', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCloakingSubsystems', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXStructuralIntegrity', true}},
				{ BOUC, 'CanUpgrade', { 'EXCompositeMaterials' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXCompositeMaterials' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXPowerBooster',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXMasor', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEmitterArray', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCapacitors', true}},
				{ BOUC, 'CanUpgrade', { 'EXAdvancedEmitterArray' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH3' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXPowerBooster' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXAdvancedEmitterArray',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXEMPArray', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCapacitors', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', true}},
				{ BOUC, 'CanUpgrade', { 'EXAdvancedEmitterArray' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH3' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXAdvancedEmitterArray' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXPowerBooster v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXMasor', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEmitterArray', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCapacitors', true}},
				{ BOUC, 'CanUpgrade', { 'EXAdvancedEmitterArray' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH3' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXPowerBooster' },
        },
    },
    Builder {
        BuilderName = 'Sorian Cybran CDR EXAdvancedEmitterArray v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXEMPArray', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCapacitors', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', true}},
				{ BOUC, 'CanUpgrade', { 'EXAdvancedEmitterArray' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH3' }},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXAdvancedEmitterArray' },
        },
    },
	
####################
########## Exaviers Seraphim ACUs
####################
##### Tier One
##########
   Builder {
        BuilderName = 'Sorian Seraphim CDR EXImprovedEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 2000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', false}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'TECH2' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'MASSEXTRACTION' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'ENERGYPRODUCTION' }},
				{ BOUC, 'CanUpgrade', { 'EXImprovedEngineering' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedEngineering' },
        },
    },
   Builder {
        BuilderName = 'Sorian Seraphim CDR EXCombatEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 2000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', false}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'TECH2' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'MASSEXTRACTION' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'ENERGYPRODUCTION' }},
				{ BOUC, 'CanUpgrade', { 'EXImprovedEngineering' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXCombatEngineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXBasicDefence',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1600,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXL1Lambda', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXL2Lambda', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXL3Lambda', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXL1Lambda' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXBasicDefence' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXL1Lambda',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1600,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXBasicDefence', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalMisslePack', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXOverchargeOverdrive', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXL1Lambda' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXL1Lambda' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXBasicDefence v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1600,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXL1Lambda', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXL2Lambda', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXL3Lambda', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXL1Lambda' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXBasicDefence' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXL1Lambda v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1600,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXBasicDefence', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalMisslePack', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXOverchargeOverdrive', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXL1Lambda' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXL1Lambda' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXCannonBigBall',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1700,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCannonRapid', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXEnergyShellHardener', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXCannonBigBall' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXCannonBigBall' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXCannonRapid',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1700,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCannonBigBall', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedContainmentBottle', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXCannonBigBall' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXCannonRapid' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXCannonBigBall v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1700,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCannonRapid', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXEnergyShellHardener', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXCannonBigBall' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXCannonBigBall' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXCannonRapid v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1700,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCannonBigBall', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedContainmentBottle', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}},
				{ BOUC, 'CanUpgrade', { 'EXCannonBigBall' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXCannonRapid' },
        },
    },
##########
##### Tier Two
##########
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXAdvancedEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1500,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', true}}, 
				{ UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'MASSEXTRACTION TECH2, MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'ENERGYPRODUCTION TECH2, ENERGYPRODUCTION TECH3' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 5, 'TECH2, TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXAdvancedEngineering' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXAdvancedEngineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXAssaultEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1500,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', true}}, 
				{ UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, 'TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'MASSEXTRACTION TECH2, MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'ENERGYPRODUCTION TECH2, ENERGYPRODUCTION TECH3' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 5, 'TECH2, TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXAdvancedEngineering' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXAssaultEngineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXTacticalMisslePack',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1200,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXL1Lambda', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXL2Lambda', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXL3Lambda', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXBasicDefence', true}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 3, 'ENERGYPRODUCTION TECH2, ENERGYPRODUCTION TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXL2Lambda' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXTacticalMisslePack' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXL2Lambda',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1200,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXBasicDefence', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalMisslePack', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXOverchargeOverdrive', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXL1Lambda', true}},
				{ BOUC, 'CanUpgrade', { 'EXL2Lambda' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXL2Lambda' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXTacticalMisslePack v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1200,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXL1Lambda', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXL2Lambda', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXL3Lambda', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXBasicDefence', true}},
				{ BOUC, 'CanUpgrade', { 'EXL2Lambda' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXTacticalMisslePack' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXL2Lambda v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1200,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXBasicDefence', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalMisslePack', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXOverchargeOverdrive', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXL1Lambda', true}},
				{ BOUC, 'CanUpgrade', { 'EXL2Lambda' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXL2Lambda' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXImprovedContainmentBottle',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1300,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCannonRapid', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXEnergyShellHardener', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCannonBigBall', true}},
                { BOUC, 'CanUpgrade', { 'EXImprovedContainmentBottle' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2, TECH3' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedContainmentBottle' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXImprovedCoolingSystem',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1300,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCannonBigBall', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedContainmentBottle', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCannonRapid', true}},
                { BOUC, 'CanUpgrade', { 'EXImprovedContainmentBottle' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2, TECH3' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedCoolingSystem' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXImprovedContainmentBottle v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1300,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCannonRapid', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXEnergyShellHardener', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCannonBigBall', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedContainmentBottle' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2, TECH3' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedContainmentBottle' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXImprovedCoolingSystem v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1300,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCannonBigBall', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedContainmentBottle', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXCannonRapid', true}},
				{ BOUC, 'CanUpgrade', { 'EXImprovedContainmentBottle' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2, TECH3' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXImprovedCoolingSystem' },
        },
    },
##########
##### Tier Three
##########
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXExperimentalEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1100,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCombatEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', true}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'ENERGYPRODUCTION TECH3' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 5, 'TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXExperimentalEngineering' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXExperimentalEngineering' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXApocolypticEngineering',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1100,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAdvancedEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXAssaultEngineering', true}},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'MASSEXTRACTION TECH3' }},
                { SIBC, 'HaveGreaterThanUnitsWithCategory', { 2, 'ENERGYPRODUCTION TECH3' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH2' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 5, 'TECH3' }},
				{ BOUC, 'CanUpgrade', { 'EXExperimentalEngineering' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXApocolypticEngineering' },
        },
    },
   	Builder {
        BuilderName = 'Sorian Seraphim CDR EXOverchargeOverdrive',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 900,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXL1Lambda', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXL2Lambda', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXL3Lambda', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalMisslePack', true}},
				{ BOUC, 'CanUpgrade', { 'EXL3Lambda' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXOverchargeOverdrive' },
        },
    },
   	Builder {
        BuilderName = 'Sorian Seraphim CDR EXL3Lambda',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 900,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXBasicDefence', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalMisslePack', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXOverchargeOverdrive', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXL2Lambda', true}},
				{ BOUC, 'CanUpgrade', { 'EXL3Lambda' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXL3Lambda' },
        },
    },
   	Builder {
        BuilderName = 'Sorian Seraphim CDR EXOverchargeOverdrive v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 900,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXL1Lambda', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXL2Lambda', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXL3Lambda', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalMisslePack', true}},
				{ BOUC, 'CanUpgrade', { 'EXL3Lambda' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXOverchargeOverdrive' },
        },
    },
   	Builder {
        BuilderName = 'Sorian Seraphim CDR EXL3Lambda v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 900,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXBasicDefence', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXTacticalMisslePack', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXOverchargeOverdrive', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXL2Lambda', true}},
				{ BOUC, 'CanUpgrade', { 'EXL3Lambda' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXL3Lambda' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXPowerBooster',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCannonRapid', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXEnergyShellHardener', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedContainmentBottle', true}},
				{ BOUC, 'CanUpgrade', { 'EXPowerBooster' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH3' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXPowerBooster' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXEnergyShellHardener',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCannonBigBall', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedContainmentBottle', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXExperimentalEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', true}},
				{ BOUC, 'CanUpgrade', { 'EXPowerBooster' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH3' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXEnergyShellHardener' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXPowerBooster v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCannonRapid', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXEnergyShellHardener', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedContainmentBottle', true}},
				{ BOUC, 'CanUpgrade', { 'EXPowerBooster' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH3' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXPowerBooster' },
        },
    },
    Builder {
        BuilderName = 'Sorian Seraphim CDR EXEnergyShellHardener v2',
        PlatoonTemplate = 'CommanderEnhanceSorian',
		Priority = 1000,
        BuilderConditions = {
				{ SBC, 'CmdrHasUpgrade', { 'EXCannonBigBall', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedContainmentBottle', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXPowerBooster', false}},
				{ SBC, 'CmdrHasUpgrade', { 'EXApocolypticEngineering', true}},
				{ SBC, 'CmdrHasUpgrade', { 'EXImprovedCoolingSystem', true}},
				{ BOUC, 'CanUpgrade', { 'EXPowerBooster' }},
				{ SIBC, 'HaveGreaterThanUnitsWithCategory', { 10, 'TECH3' }},
                { MIBC, 'FactionIndex', {4, 4}},
            },
        PlatoonAddFunctions = { {SAI, 'BuildOnce'}, },
        BuilderType = 'Any',
        BuilderData = {
            Enhancement = { 'EXEnergyShellHardener' },
        },
    },
}

end